#
# $HeadURL: https://svn.oucs.ox.ac.uk/sysdev/src/packages/c/configtool/tags/2.12-1~ox.0/lib/Configtool/Config.pm $
# $LastChangedRevision: 16104 $
# $LastChangedDate: 2009-08-28 17:31:39 +0100 (Fri, 28 Aug 2009) $
# $LastChangedBy: tom $
#
package Configtool::Config;

use strict;
use warnings FATAL => 'all';

use base 'AppConfig';

use Readonly;

Readonly my %CONFIG => ( lockpath        => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                              DEFAULT  => '/var/lock/configtool'
                                          },
                         disablepath     => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                              DEFAULT  => '/var/lib/configtool/configtool.disable'
                                          },
                         deletepath      => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                              DEFAULT  => '/etc/configtool.delete'
                                          },
                         metapath        => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                              DEFAULT  => '/var/lib/configtool/repository/etc/configtool.meta'
                                          },
                         verbose         => { ARGCOUNT => AppConfig::ARGCOUNT_NONE
                                          },
                         test            => { ARGCOUNT => AppConfig::ARGCOUNT_NONE,
                                              ALIAS    => 'dry-run|dryrun'
                                          },
                         disable         => { ARGCOUNT => AppConfig::ARGCOUNT_ONE
                                          },
                         enable          => { ARGCOUNT => AppConfig::ARGCOUNT_NONE
                                          },
                         force           => { ARGCOUNT => AppConfig::ARGCOUNT_NONE
                                          },
                         sync            => { ARGCOUNT => AppConfig::ARGCOUNT_NONE
                                          },
                         pre             => { ARGCOUNT => AppConfig::ARGCOUNT_NONE,
                                              DEFAULT  => 1
                                          },
                         post            => { ARGCOUNT => AppConfig::ARGCOUNT_NONE,
                                              DEFAULT  => 1
                                          },
                         sync_userskel   => { ARGCOUNT => AppConfig::ARGCOUNT_NONE,
                                              ALIAS    => 'sync-userskel',
                                          },
                         delete          => { ARGCOUNT => AppConfig::ARGCOUNT_NONE,
                                          },
                         mkdirs          => { ARGCOUNT => AppConfig::ARGCOUNT_NONE,
                                              DEFAULT  => 1
                                          },
                         showdiff        => { ARGCOUNT => AppConfig::ARGCOUNT_NONE,
                                              DEFAULT  => 1,
                                              ALIAS    => 'show-diff'
                                          },
                         sync_class      => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                              DEFAULT  => 'Configtool::Sync::Rsync'
                                          },
                         sync_option     => { ARGCOUNT => AppConfig::ARGCOUNT_HASH,
                                          },
                         repository      => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                              DEFAULT  => '/var/lib/configtool/repository'
                                          },
                         userskel        => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                              DEFAULT  => '/var/lib/configtool/userskel'
                                          },
                         userskel_class  => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                              DEFAULT  => 'Configtool::Sync::Subversion'
                                         },
                         userskel_option => { ARGCOUNT => AppConfig::ARGCOUNT_HASH,
                                          },
                         ignore_no_files_error
                                         => { ARGCOUNT => AppConfig::ARGCOUNT_NONE,
                                              ALIAS    => 'ignore-no-files-error'
                                          },
                         users_conf      => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                              DEFAULT  => '/etc/configtool/users.conf',
                                              ALIAS    => 'users-conf',
                                          },
                         update_users    => { ARGCOUNT => AppConfig::ARGCOUNT_NONE,
                                              ALIAS    => 'update-users'
                                          },
                         sys_user        => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                              DEFAULT  => 'configtool',
                                              ALIAS    => 'sys-user',
                                          },
                         rootdir      => { ARGCOUNT => AppConfig::ARGCOUNT_ONE,
                                           DEFAULT  => "/"
                                       },

                     );

sub new {
    my $proto = shift;
    my $class = ref( $proto ) || $proto;
    my $self = $class->SUPER::new( { PEDANTIC => 1,
                                     ERROR    => sub { die sprintf( shift, @_ ) . "\n" } } );

    while ( my ( $k, $v ) = each %CONFIG ) {
        $self->define( $k => $v );
    }

    bless( $self, $class );
}

1;

__END__
