#
# $HeadURL: https://svn.oucs.ox.ac.uk/sysdev/src/packages/c/configtool/tags/2.12-1~ox.0/lib/Configtool/File.pm $
# $LastChangedRevision: 16104 $
# $LastChangedDate: 2009-08-28 17:31:39 +0100 (Fri, 28 Aug 2009) $
# $LastChangedBy: tom $
#
package Configtool::File;

use strict;
use warnings FATAL => 'all';

use Class::Std;
use Contextual::Return;
use User::pwent;
use User::grent;

{
    my %path_for  :ATTR( :name<path>                 );
    my %src_for   :ATTR( :name<src>                  );
    my %owner_for :ATTR( :name<owner> :default<root> );
    my %group_for :ATTR( :name<group> :default<root> );
    my %pre_for   :ATTR( :name<pre>   :default<>     );
    my %post_for  :ATTR( :name<post>  :default<>     );
    my %mode_for  :ATTR(                             );
    my %uid_for   :ATTR(                             );
    my %gid_for   :ATTR(                             );

    sub BUILD {
        my ( $self, $obj_id, $arg_ref ) = @_;

        my $mode = defined( $arg_ref->{ mode } ) ? $arg_ref->{ mode } : '0444';

        $self->set_mode( $mode );
    }

    sub set_mode {
        my ( $self, $mode ) = @_;

        $mode_for{ $self->ident } = oct( $mode );
    }

    sub get_mode {
        my ( $self ) = @_;

        my $mode = $mode_for{ $self->ident };

        return SCALAR { $mode }
                  NUM { $mode }
                  STR { sprintf( '0%o', $mode ) };
    }

    sub get_uid {
        my $self = shift;

        unless ( defined $uid_for{ $self->ident } ) {
            my $pwent = getpwnam( $self->get_owner )
                or die "No such user " . $self->get_owner;
            $uid_for{ $self->ident } = $pwent->uid;
        }

        return $uid_for{ $self->ident };
    }

    sub get_gid {
        my $self = shift;

        unless ( defined $gid_for{ $self->ident } ) {
            my $grent = getgrnam( $self->get_group )
                or die "No such group " . $self->get_group;
            $gid_for{ $self->ident } = $grent->gid;
        }

        return $gid_for{ $self->ident };
    }

    sub as_str :STRINGIFY {
        my $self = shift;
        $self->get_path();
    }
}

1;

__END__
