#
# $HeadURL: https://svn.oucs.ox.ac.uk/sysdev/src/packages/c/configtool/tags/2.12-1~ox.0/lib/Configtool/Manifest.pm $
# $LastChangedRevision: 16104 $
# $LastChangedDate: 2009-08-28 17:31:39 +0100 (Fri, 28 Aug 2009) $
# $LastChangedBy: tom $
#
package Configtool::Manifest;

use strict;
use warnings FATAL => 'all';

use Class::Std;
use Configtool::File;
use File::Find;
use File::Spec;
use IO::File;
#use Smart::Comments;

{
    my %repository_for :ATTR( :init_arg<repository> );
    my %metadata_for   :ATTR( :init_arg<metadata>   );
    my %files_for      :ATTR( :get<files>           );
    my %rootdir_for   :ATTR( :name<rootdir>       );

    sub START {
        my ( $self, $obj_id, $arg_ref ) = @_;

        $files_for{ $obj_id } = $self->init_files( $repository_for{ $obj_id },
                                                   $metadata_for{ $obj_id } );
    }

    sub get_matching_files {
        my ( $self, @wanted ) = @_;

        my $rx = '^(' . join( q{|}, map qr/\Q$_\E/, map { s{/$}{}; $_ } @wanted ) . ')(/|$)';
        ### $rx

        my @matched;

        foreach my $f ( @{ $self->get_files } ) {
            if ( $f->get_path =~ $rx ) {
                push @matched, $f;
            }
        }

        return \@matched;
    }

    sub init_files :PRIVATE {
        my ( $self, $repos_path, $meta_path ) = @_;

        my $meta = $self->parse_metadata( $meta_path );

        my @files;

        File::Find::find( sub {
                              if ( -d and $_ eq '.svn' ) {
                                  $File::Find::prune = 1;
                                  return;
                              }
                              if ( -f and not /\.prect$/ and not /\.postct$/ ) {
                                  my $src  = $File::Find::name;
                                  my $path = File::Spec->catfile( $self->get_rootdir, File::Spec->abs2rel( $src, $repos_path ) );
                                  my $path_from_root = "/" . File::Spec->abs2rel( $path, $self->get_rootdir );

                                  my $this_file = Configtool::File->new( { path => $path,
                                                                           src  => $src,
                                                                         } );

                                  $this_file->set_pre( "$src.prect" )
                                      if -f "$src.prect";

                                  $this_file->set_post( "$src.postct" )
                                      if -f "$src.postct";

                                  if ( my $m = $meta->{ $path_from_root } ) {
                                      $this_file->set_owner( $m->{owner} )
                                          if defined $m->{owner};
                                      $this_file->set_group( $m->{group} )
                                          if defined $m->{group};
                                      $this_file->set_mode( $m->{mode} )
                                          if defined $m->{mode};
                                  }

                                  push @files, $this_file;
                              }
                          },
                          $repos_path
                      );

        return \@files;
    }

    sub parse_metadata :PRIVATE {
        my ( $self, $meta_path ) = @_;

        my %meta;

        my $ifh = IO::File->new( $meta_path, O_RDONLY )
            or die "open $meta_path: $!";

        while ( <$ifh> ) {
            next unless /\S/ and not /^#/;
            chomp;
            my ( $path, $owner, $group, $mode ) = split;
            next if $path =~ /\.(pre|post)ct/;
            @{ $meta{ $path } }{ qw(owner group mode ) } = ( $owner, $group, $mode );
        }

        return \%meta;
    };

}

1;

__END__
