#
# $HeadURL: https://svn.oucs.ox.ac.uk/sysdev/src/packages/c/configtool/tags/2.12-1~ox.0/lib/Configtool/Sync/Subversion.pm $
# $LastChangedRevision: 16104 $
# $LastChangedDate: 2009-08-28 17:31:39 +0100 (Fri, 28 Aug 2009) $
# $LastChangedBy: tom $
#
package Configtool::Sync::Subversion;

use strict;
use warnings FATAL => 'all';

use File::Path qw( rmtree );
use Readonly;

Readonly my @SVN      => qw( /usr/bin/svn );
Readonly my @SVN_OPTS => qw( config-dir username password non-interactive no-auth-cache );
Readonly my $QUIET    => '--quiet';

sub sync {
    my ( $class, $config ) = @_;

    my $src     = $config->sync_option->{ source };
    my $dest    = $config->repository;

    die "subversion sync source URL not specified\n"
        unless defined $src;

    _svn_sync( $src, $dest, $config->sync_option, $config->verbose );
}

sub sync_userskel {
    my ( $class, $config ) = @_;

    my $src     = $config->userskel_option->{ source };
    my $dest    = $config->userskel;

    die "subversion userskel source URL not specified\n"
        unless defined $src;

    _svn_sync( $src, $dest, $config->userskel_option, $config->verbose );
}

sub _svn_sync {
    my ( $src, $dest, $options, $verbose ) = @_;

    print "Syncing $src to $dest\n"
        if $verbose;

    my @cmd = @SVN;
    foreach my $o ( @SVN_OPTS ) {
        if ( exists $options->{ $o } ) {
            my $value = $options->{ $o };
            push @cmd, "--$o";
            push @cmd, $value
                if defined $value;
        }
    }
    push @cmd, $QUIET
        unless $verbose;

    if ( -d "$dest/.svn" ) {
        push @cmd, 'update', $dest;
    }
    else {
        rmtree( $dest );
        push @cmd, 'checkout', $src, $dest;
    }

    system( @cmd ) == 0
        or die "failed to run @cmd: exit $?\n";
}

1;

__END__
